// This is now a rule to have ALL global variables to have a "gfx_" prefix.
// This is done to minimize collision with other names.

struct VSInput
{
	float3 position : POSITION0;
	float3 normal	: NORMAL0;
};

struct VSOutput
{
	float4 positionCS	: SV_Position;
	float3 positionWS	: POSITIONWS;
	float3 normalWS		: NORMALWS;
};

struct PSInput
{
	float4 positionCS	: SV_Position;
	float3 positionWS	: POSITIONWS;
	float3 normalVS		: NORMALVS;
};

struct PSOutput
{
	float3 normal : SV_Target0;
};

///////////////////////////////////////////////////////////

cbuffer cPerObject
{
	matrix gfx_gWorld;
	matrix gfx_gWorldView;
	matrix gfx_gWorldViewProj;
};

///////////////////////////////////////////////////////////

VSOutput VS( in VSInput input )
{
	VSOutput output;
	float4 pos = float4( input.position, 1.0f );

	// Transform to homogenous clip space
	output.positionCS = mul(pos, gfx_gWorldViewProj );
	
	// Transform to world space
	output.positionWS = mul( pos, gfx_gWorld );

	// Transform to view space.
	output.normalVS = normalize( mul( input.normal, (float3x3)gfx_gWorldView ) );

	return output;
}

PSOutput PS( in PSInput input )
{
	PSOutput output;

	output.normal = normalize( input.normalVS );

	return output;
};

