/**************************************************************************************
*
*	Copyright (C) 2010 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	- AtomicInt.h
*
*	Description - 
*
*	Comments	-
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	05/21/2010	-	Creation of this file.
**************************************************************************************/
#ifndef _PSX_ATOMIC_INT_H_
#define _PSX_ATOMIC_INT_H_

#include "PulseSTD.h"
#include "Atomic.h"

namespace Pulse
{
	class AtomicInt
	{
	public:

		AtomicInt( INT val = 0 );

		INT operator ++ ( INT );

		INT operator -- ( INT );

		INT operator ++ ( void );

		INT operator -- ( void );

		void operator = ( INT a_iExchange );

		INT CompareExchange( INT a_iComperand, INT a_iExchange );

		INT GetValue( void ) const;

		operator INT( void ) const;

	private:

		// Atomically modifed value. Declared mutable such that function that emulate atomic 
		// reads can be implemented using standard _InterlockedXXX functions 
		mutable volatile LONG m_val;

	};

	PSX_FORCE_INLINE AtomicInt::AtomicInt( INT val ) 
		: m_val(val)
	{
	}

	PSX_FORCE_INLINE INT AtomicInt::operator ++ ( INT )
	{
		return _InterlockedExchangeAdd( &m_val, 1 );
	}

	PSX_FORCE_INLINE INT AtomicInt::operator -- ( INT )
	{
		return _InterlockedExchangeAdd(&m_val, -1);
	}

	PSX_FORCE_INLINE INT AtomicInt::operator ++ ( void )
	{
		return _InterlockedIncrement( &m_val );
	}

	PSX_FORCE_INLINE INT AtomicInt::operator -- ( void )
	{
		return _InterlockedDecrement( &m_val );
	}

	PSX_FORCE_INLINE void AtomicInt::operator = ( INT a_iExchange )
	{
		_InterlockedExchange( &m_val, a_iExchange );
	}

	PSX_FORCE_INLINE INT AtomicInt::CompareExchange( INT a_iComperand, INT a_iExchange )
	{
		return _InterlockedCompareExchange( &m_val, a_iExchange, a_iComperand );
	}

	PSX_FORCE_INLINE INT AtomicInt::GetValue( void ) const
	{
		return _InterlockedExchangeAdd( &m_val, 0 );
	}

	PSX_FORCE_INLINE AtomicInt::operator INT( void ) const
	{
		return GetValue();
	}

}

#endif /* _PSX_ATOMIC_INT_H_ */