/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	IParameter.h
*
*	Description -	A single parameter object used for the rendering parameter system.
*
*	Comments	-
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	12/15/2011	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_RENDER_PARAMETER_H_
#define _PSX_RENDER_PARAMETER_H_

#include "PulseSTD.h"
#include "StringID.h"

namespace Pulse
{
	PSX_EnumBegin( EParameter )
		UNKNOWN = -1,
		SCALAR,
		VECTOR3,
		VECTOR4,
		MATRIX,
		MATRIX_ARRAY,
		SHADER_RESOURCE,
		UNORDERED_ACCESS,
		CONSTANT_BUFFER,
		STRUCTURED_BUFFER,
		SAMPLER,
		TEXTURE2D,
		TEXTURE3D,
		ENTITY,	// I don't know what this is...
	PSX_EnumEnd()

	class IParameter
	{
	public:

		IParameter( void );

		IParameter( IParameter &rhs );

		virtual ~IParameter() { }

		IParameter * CreateCopy( void );

		void SetName( const CHAR *pName ) { m_name.SetString( pName ); }

		const CHAR * GetName( void ) const { return m_name.GetString(); }

		const UINT GetID( void ) const { return m_name.GetID(); }

		virtual void SetData( void *pData ) = 0;

		// Derived classes should be careful about this function. Make sure
		//	pParam is the same type as the data being written to.
		virtual void SetData( IParameter *pParam ) = 0;

		virtual EParameter::Type GetType( void ) const = 0;

		// Convenience function to let subclasses be referred by their parent
		//	to update their internal values.
		//virtual void UpdateValue( IParameter* pParameter ) = 0;

		const UINT GetChangeValue( void ) const { return m_changeValue; }

	protected:

		void IncrementChangeValue( void ) { ++m_changeValue; }

		void SetChangeValue( UINT value ) { m_changeValue = value; }

		StringID m_name;

		// This gets incremented everytime the binder changes the value
		UINT	m_changeValue; 
	};

}

#endif /* _PSX_RENDER_PARAMETER_H_ */