/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	ParameterBinder.h
*
*	Description -	Binds values to parameter objects given by ParameterSystem.
*
*
*	Comments	-
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	06/10/2012	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_PARAMETER_BINDER_H_
#define _PSX_PARAMETER_BINDER_H_

#include "PulseSTD.h"
#include "IParameter.h"

namespace Pulse
{
	class IParameterBinder
	{
	public:

		IParameterBinder( void );

		virtual ~IParameterBinder( void  );

		virtual void SetAsScalar( const BOOL val );

		virtual void SetAsScalar( const INT val );

		virtual void SetAsScalar( const UINT val );

		virtual void SetAsScalar( const FLOAT val );

		virtual void SetAsMatrix( const Matrix4x4 *pData );

		virtual void SetAsMatrixArray( const Matrix4x4 *pData, SIZE_T arraySize );

		//virtual void SetAsShaderResource( ShaderResource *pData );
		//virtual void SetAsShaderResource( ShaderResource *pData );	// TODO: Make one for Texture1D
		virtual void SetAsShaderResource( Texture2D *pTexture );
		//virtual void SetAsShaderResource( ShaderResource *pData );	// TODO: Make one for Texture3D

		virtual void SetAsVector3( const Vector3 *pData );

		virtual void SetAsVector4( const Vector4 *pData );

		virtual void SetAsSamplerState( SamplerState *pState );

		virtual void SetAsConstantBuffer( ConstantBuffer *pBuffer );

		IParameter * GetParameter( void ) { return m_pParam; }

		virtual void SetParameter( IParameter *pParam );

		virtual void BindValue( void ) = 0;

		virtual EParameter::Type GetType( void ) = 0;

	protected:

		union
		{
			IParameter				*m_pParam;

			// For easy usage of IParameter in derived classes.
			ParameterScalar			*m_pScalarParam;
			ParameterMatrix			*m_pMatrixParam;
			ParameterMatrixArray	*m_pMatrixArrayParam;
			ParameterShaderResource *m_pShaderResourceParam;
			ParameterVector3		*m_pVector3Param;
			ParameterVector4		*m_pVector4Param;
			ParameterConstantBuffer *m_pConstantBufferParam;
			ParameterSamplerState	*m_pSamplerStateParam;
		};

	};
}

#endif /* _PSX_PARAMETER_BINDER_H_ */