/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	IShaderStage.h
*
*	Description -	A customizable stage in the graphics pipeline that runs using shaders.
*
*	Comments	-	Base class.
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	10/19/2011	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_SHADER_STAGE_H_
#define _PSX_SHADER_STAGE_H_

#include "PulseSTD.h"
#include "GraphicsInfrastructure.h"
#include "GraphicsTypes.h"
#include "IPipelineStage.h"
#include "ShaderStageState.h"

namespace Pulse
{
	class IShaderStage : public IPipelineStage
	{
	public:

		IShaderStage( void );

		virtual ~IShaderStage( void );

		void SetShader( Shader *pShader );
		void SetConstantBuffer( UINT slot, const ConstantBuffer *pConstantBuffer );
		void SetSamplerState( UINT slot, const SamplerState *pSampler );
		void SetShaderResource( UINT slot, const ShaderResource *pGraphicsResource );
		void SetUnordererdAccess( UINT slot, const ShaderResource *pGraphicsResource );

		virtual void ClearStates( void );
		virtual void BindStates( void );

		virtual void BindShader( void ) { PSX_PushError("BindShader not implemented in this Shader Stage."); }
		virtual void BindConstantBuffers( void ) { PSX_PushError("BindConstantBuffers not implemented in this Shader Stage."); }
		virtual void BindSamplerStates( void ) { PSX_PushError("BindSamplerStates not implemented in this Shader Stage."); }
		virtual void BindShaderResourceViews( void ) { PSX_PushError("BindShaderResourceViews not implemented in this Shader Stage."); }
		virtual void BindUnordererdAccessViews( void ) { PSX_PushError("BindUnordererdAccessViews not implemented in this Shader Stage."); }

	protected:

		ShaderStageState m_stateCache;

	};
}

#endif /* _PSX_SHADER_STAGE_H_ */