/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	IndexBuffer.h
*
*	Description -	Index buffer
*
*	Comments	-	
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	11/23/2011	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_INDEX_BUFFER_H_
#define _PSX_INDEX_BUFFER_H_

#include "PulseSTD.h"
#include "GraphicsTypes.h"
#include "ResourcePoolItem.h"
#include "ShaderResource.h"
#include "GraphicsBuffer.h"

namespace Pulse
{
	struct IndexBufferDesc : public BaseResDesc
	{
		IndexBufferDesc( void );

		void SetAsDynamicIndexBuffer( UINT _numIndices, void *pInitialData = PSX_NULL );
		
		void SetAsStaticIndexBuffer( UINT _numIndices, void *pInitialData, BOOL bUse16Bit = TRUE );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::INDEX_BUFFER ); }

		UINT					indexSize;
		UINT					numIndices;
		EBufferUsage::Type		bufferUsage;
		void					*pData;

	};

	class IndexBuffer : public IResourcePoolItem, /*public ShaderResource,*/ public GraphicsBuffer
	{
	public:

		virtual ~IndexBuffer( void );

		// These defines the base interface for all derived resources
		virtual BOOL CreateResource( BaseResDesc *pDesc );
		virtual BOOL DestroyResource( void );
		virtual BOOL LoadResource( void );	
		virtual BOOL UnloadResource( void );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::INDEX_BUFFER ); }

		void * GetNakedIndexBuffer( void ) { return (void*)GetNakedBuffer(); }

		IndexBufferDesc * GetDescription( void ) { return &m_desc; }

	private:

		IndexBuffer( void );

		EErrorCode::Type Initialize( void );

		void Cleanup( void );

	private:

		PSX_MakeStoragePoolFriend( IndexBuffer );

		IndexBufferDesc		m_desc;

	};
}

#endif /* _PSX_INDEX_BUFFER_H_ */