/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	RenderPass.h
*
*	Description -	
*
*	Comments	-	
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	06/05/2012	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_RENDER_PASS_H_
#define _PSX_RENDER_PASS_H_

#include "PulseSTD.h"
#include "GraphicsInfrastructure.h"
#include "ResourcePoolItem.h"
#include "VertexShader.h"
#include "HullShader.h"
#include "DomainShader.h"
#include "GeometryShader.h"
#include "PixelShader.h"
#include "ComputeShader.h"
#include "Array.h"
#include "MaterialElementInfo.h"
#include "GeometryElementInfo.h"

namespace Pulse
{
	class RenderPass
	{
	public:

		RenderPass( void );

		RenderPass( VertexShader *pVS, PixelShader *pPS );

		virtual ~RenderPass( void ); 

		void SetRenderViewStage( ERenderViewStage::Type viewStage ) { m_renderViewStage = viewStage; }
		const ERenderViewStage::Type GetRenderViewStage( void ) const { return m_renderViewStage; }

		void SetVertexShader( VertexShader *pShader );
		void SetVertexShader( Shader *pShader );

		void SetHullShader( HullShader *pShader );
		void SetHullShader( Shader *pShader );

		void SetDomainShader( DomainShader *pShader );
		void SetDomainShader( Shader *pShader );

		void SetGeometryShader( GeometryShader *pShader );
		void SetGeometryShader( Shader *pShader );

		void SetPixelShader( PixelShader *pShader );
		void SetPixelShader( Shader *pShader );

		void SetComputeShader( ComputeShader *pShader );
		void SetComputeShader( Shader *pShader );

		Shader * GetShader( EShader::Type type );

		void SetBlendState( BlendState *pState );

		void SetDepthStencilState( DepthStencilState *pState );

		void SetRasterizerState( RasterizerState *pState );

		const SIZE_T GetNumGeometryRequirements( void ) const { return m_geometryRequirements.GetSize(); }

		GeometryElementRequirement * GetGeometryRequirements( void );

		const SIZE_T GetNumMaterialRequirements( void ) const { return m_materialRequirements.GetSize(); }

		MaterialElementRequirement * GetMaterialRequirements( void );

		void Update( void );

		void ConfigurePipeline( GraphicsPipeline *pPipeline );

	private:

		friend Effect;
		friend RenderTechnique;

		void ClearShaders( bool bCheckRelease );

		void AddGeometryRequirement( GeometryElementRequirement *pRequirement );

		void AddMaterialRequirement( MaterialElementRequirement *pRequirement );

	private:

		typedef Array<GeometryElementRequirement> GeometryRequirementList;
		typedef Array<MaterialElementRequirement> MaterialRequirementList;

		union
		{
			Shader *m_pShaders[EShader::EnumCount];

			struct
			{
				VertexShader	*m_pVertexShader;
				HullShader		*m_pHullShader;
				DomainShader	*m_pDomainShader;
				GeometryShader	*m_pGeometryShader;
				PixelShader		*m_pPixelShader;
				ComputeShader	*m_pComputeShader;
			};
		};

		ERenderViewStage::Type		m_renderViewStage;
		DepthStencilState			*m_pDepthStencilState;
		BlendState					*m_pBlendState;
		RasterizerState				*m_pRasterizerState;
		GeometryRequirementList		m_geometryRequirements;
		MaterialRequirementList		m_materialRequirements;
	};
}

#endif /* _PSX_RENDER_PASS_H_ */