/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	Renderer.h
*
*	Description -	Graphics Renderer.
*
*	Comments	-
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	10/06/2011	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_RENDERER_H_
#define _PSX_RENDERER_H_

#include "PulseSTD.h"
#include "GraphicsTypes.h"
#include "GraphicsDevice.h"
#include "GraphicsPipeline.h"
#include "ParameterSystem.h"
#include "RenderView.h"
#include "RenderPathForwardPlus.h"

namespace Pulse
{
	struct RendererDesc
	{
		
		void SetAsImmediate( ERendererProcessingType::Type	_processingType, UINT _adapter, UINT _adapterOutput, BOOL _bDebug,
			BOOL _bSoftwareRendererFallback, DisplayMode _displaySettings );
		
		void SetAsDeferred( void );

		ERendererProcessingType::Type	processingType;
		EDeviceContext::Type			deviceContextType;
		UINT							adapter;
		UINT							adapterOutput; // Device to output to such as a monitor.
		BOOL							bDebug;
		BOOL							bSoftwareRendererFallback;
		DisplayMode						displaySettings;
	};

	class Renderer
	{
	public:

		Renderer( void );

		virtual ~Renderer( void );

		EErrorCode::Type Initialize( RendererDesc  *pDesc );

		// HACK: Post initialization after initialize...
		EErrorCode::Type InitializeRenderPath( void );

		void Cleanup( void );

		GraphicsDevice * GetDevice( void ) { return m_pDevice; }

		void * GetNakedDevice( void ) { return m_pDevice->GetNakedDevice(); }

		EDeviceContext::Type GetDeviceContextType( void ) const { return m_desc.deviceContextType; }

		void * GetNakedDeviceContext( void ) { return (void*)m_pDeviceContext; }
		
		// NOTE: This may not be used. This will be deprecated.
		MappedResource MapResource( GraphicsBuffer *pBuffer, UINT subresourceIndex, EResourceMapping::Type map, UINT flags = 0 );

		void UnmapResource( GraphicsBuffer *pBuffer, UINT subresourceIndex );

		GraphicsPipeline * GetPipeline( void ) { return m_pPipeline; }

		ParameterSystem * GetParameterSystem( void ) { return &m_parameters; }

		EErrorCode::Type LoadEffectsLibrary( const CHAR *pFile );

		RenderPathForwardPlus * GetRenderPath( void ) { return &m_renderPath; }

		//////////////////////////////////////////
		// NOTE: These method's names may be changed in the future to better
		//	reflect their behaviors.
		void AddToRender( Model *pModel );

		void AddToRender( RenderEntry *pEntry );

		void BeginRender( void );

		void Render( void );

		void EndRender( void );

		void Present( void );

		void ExecuteRenderStages( void );

		void ExecuteRenderEntry( RenderEntry *pEntry );
		//////////////////////////////////////////

	private:

		friend GraphicsPipeline;

		// Internal methods used by the graphics pipeline
		void DrawIndexed( UINT indexCount, UINT startIndexLocation = 0, INT baseVertexLocation = 0 );

		void Dispatch( UINT threadCountX, const UINT threadCountY, const UINT threadCountZ );

	private:


		RendererDesc		m_desc;
		GraphicsDevice		*m_pDevice;
		ID3D11DeviceContext *m_pDeviceContext; // This can be deferred or immedate

		GraphicsPipeline	*m_pPipeline;

		ParameterSystem		m_parameters;

		EffectsLibrary *m_pEffectsLibrary;

		// We can only have one render path set at a time
		//	so place it here.
		RenderPathForwardPlus m_renderPath;

	};
}

#endif /* _PSX_RENDERER_H_ */