/**************************************************************************************
*
*	Copyright (C) 2010 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	ResourceManager.h
*
*	Description -	Resource management system.
*
*	Comments	-	
*					
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	04/25/10	-	Creation of this file
**************************************************************************************/

#ifndef _PSX_RESOURCE_MANAGER_H_
#define _PSX_RESOURCE_MANAGER_H_

#include "PulseSTD.h"
#include "Singleton.h"
#include "ResourcePool.h"
#include "ResourcePoolItem.h"

namespace Pulse
{
	class ResourceManager : public Singleton< ResourceManager >
	{
	public:

		~ResourceManager( void );

		// Resource item management interface
		void RegisterResourcePool( IResourcePool *pResPool, ResourceCode code );
		IResourcePool * UnregisterResourcePool( ResourceCode code );

		void CleanAll( void );
		void DestroyAll( void );
		void DisableAll( void );
		void RestoreAll( void );

		void CleanResourceType( EResource::Type type );
		void DestroyResourceType( EResource::Type type );
		void DisableResourceType( EResource::Type type );
		void RestoreResrouceType( EResource::Type type );

		IResourcePool * FindResourcePool( ResourceCode code );
		HRes FindResourceHandle( ResourceCode code, const String *pName );
		IResourcePoolItem * FindResource( ResourceCode code, const String *pName );
		IResourcePoolItem * FindResource( ResourceCode code, HRes hRes );

	private:

		friend Singleton;

		explicit ResourceManager( void ) { }
		explicit ResourceManager( ResourceManager & ) { }
		ResourceManager & operator = ( ResourceManager & ) { }

		typedef Map< ResourceCode, IResourcePool* > ResourcePoolTypeMap;
		typedef ResourcePoolTypeMap::Iterator MapIter;
		typedef List< IResourcePool* > ResourcePoolList;
		typedef ResourcePoolList::Iterator ListIter;

		ResourcePoolList m_resPoolList[EResource::EnumCount];
		ResourcePoolTypeMap m_resPoolMap;

	};

	PSX_INLINE ResourceManager::~ResourceManager( void )
	{
		DestroyAll();
	}

	PSX_INLINE void ResourceManager::RegisterResourcePool( IResourcePool *pResPool, ResourceCode code )
	{
		m_resPoolMap[ code ] = pResPool;
		m_resPoolList[ code.type ].PushBack( pResPool );
	}

	PSX_INLINE void ResourceManager::CleanAll( void )
	{
		for ( INT i = 0; i < EResource::EnumCount; ++i )
			CleanResourceType( (EResource::Type)i );
	}

	PSX_INLINE void ResourceManager::DestroyAll( void )
	{
		for ( INT i = 0; i < EResource::EnumCount; ++i )
			DestroyResourceType( (EResource::Type)i );
	}

	PSX_INLINE void ResourceManager::DisableAll( void )
	{
		for ( INT i = 0; i < EResource::EnumCount; ++i )
			DisableResourceType( (EResource::Type)i );
	}

	PSX_INLINE void ResourceManager::RestoreAll( void )
	{
		for ( INT i = 0; i < EResource::EnumCount; ++i )
			RestoreResrouceType( (EResource::Type)i );
	}

}

#endif /* _PSX_RESOURCE_MANAGER_H_ */