/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	SmartPointer.inl
*
*	Comments	-	See SmartPointer.h
*
**************************************************************************************/

namespace Pulse
{

	/* SmartData Implementation */

	// Don't implement this
	//template < typename T > 
	//PSX_FORCE_INLINE SmartPointer< T >::SmartData::SmartData() 
	//	: m_pData( 0 ), m_refCount( 0 ) 
	//{
	//
	//}

	template < typename T >
	PSX_FORCE_INLINE SmartPointer< T >::SmartData::SmartData( const T *pData ) 
		: m_pData( (T*)pData ), m_refCount( 1 ), m_weakCount( 1 )
	{

	}

	template < typename T >
	PSX_FORCE_INLINE void SmartPointer< T >::SmartData::AddRef()
	{
		++m_refCount;
	}

	template < typename T >
	void SmartPointer< T >::SmartData::Release()
	{
		PSX_Assert( m_refCount > 0, "Smart pointer already 0 and still releasing! WTF!!!!" );

		if ( --m_refCount == 0 )
		{
			DeleteData();
			ReleaseWeakRef();
		}
	}

	template < typename T >
	PSX_FORCE_INLINE void SmartPointer< T >::SmartData::AddWeakRef()
	{
		++m_weakCount;
	}

	template < typename T >
	void SmartPointer< T >::SmartData::ReleaseWeakRef()
	{
		PSX_Assert( m_weakCount > 0, "Smart pointer weak count already 0 and still releasing! WTF!!!!" );

		if ( --m_weakCount == 0 )
		{
			delete this;
		}
	}

	template < typename T >
	PSX_FORCE_INLINE T * SmartPointer< T >::SmartData::GetData()
	{
		return m_refCount ? m_pData : PSX_NULL;
	}

	template < typename T >
	PSX_FORCE_INLINE const T * SmartPointer< T >::SmartData::GetData() const
	{
		return m_refCount ? m_pData : PSX_NULL;
	}

	//template < typename T >
	//PSX_FORCE_INLINE void SmartPointer< T >::SmartData::SetData( T *pData )
	//{
	//	m_pData = pData;
	//}

	template < typename T >
	PSX_FORCE_INLINE void SmartPointer< T >::SmartData::DeleteData( void )
	{
		delete ((T*)m_pData);
		m_pData = PSX_NULL;
	}

	/* SmartPointer Implementation */

	template < typename T >
	PSX_FORCE_INLINE SmartPointer< T >::SmartPointer( void ) 
		: m_pSmartObj(0) 
	{
	
	}

	template < typename T >
	SmartPointer< T >::SmartPointer( const SmartPointer< T > &smartPointer )
	{
		m_pSmartObj = smartPointer.m_pSmartObj;
		if ( smartPointer.IsNull() == FALSE )
			AddRef();
	}

	template < typename T >
	PSX_FORCE_INLINE SmartPointer< T >::SmartPointer( const T *pData )
	{
		// Only create smart data if pData actually points to something
		//	other than NULL.
		if ( pData )
			m_pSmartObj = new SmartData( pData ); // auto sets ref to one
		else
			m_pSmartObj = NULL;
	}

	template < typename T >
	PSX_INLINE SmartPointer<T>::SmartPointer( SmartData &smartData )
		: m_pSmartObj( &smartData )
	{
			AddRef();
	}

	template < typename T >
	PSX_FORCE_INLINE SmartPointer< T >::~SmartPointer()
	{
		Release();
	}

	template < typename T >
	void SmartPointer< T >::Release()
	{
		if ( m_pSmartObj )
		{
			m_pSmartObj->Release(); 
			m_pSmartObj = NULL;
		}
	}

	template < typename T >
	PSX_INLINE void SmartPointer< T >::AddRef()
	{
		if ( m_pSmartObj )
			m_pSmartObj->AddRef();
	}
	
	template < typename T >
	PSX_INLINE const T & SmartPointer< T >::operator * ( void ) const
	{
		// This will just crash if access null pointer
		return *m_pSmartObj->GetData();
	}

	template < typename T >
	PSX_INLINE T & SmartPointer< T >::operator * ( void )
	{
		// This will just crash if access null pointer
		return *m_pSmartObj->GetData();
	}

	template < typename T >
	PSX_INLINE T * SmartPointer< T >::operator -> ( void ) const
	{
		return m_pSmartObj ? m_pSmartObj->GetData() : PSX_NULL;
	}

	template < typename T >
	PSX_INLINE SmartPointer< T >::operator T * ( void ) const
	{
		return m_pSmartObj ? m_pSmartObj->GetData() : PSX_NULL;
	}

	template < typename T >
	PSX_INLINE SmartPointer< T >::operator T & ( void ) const
	{
		return *m_pSmartObj->GetData();
	}

	template < typename T >
	PSX_INLINE BOOL SmartPointer< T >::operator == ( const SmartPointer< T > &rhs ) const
	{
		return m_pSmartObj == rhs.m_pSmartObj;
	}

	template < typename T >
	SmartPointer< T > & SmartPointer< T >::operator = ( const SmartPointer< T > &rhs )
	{
		if ( this == &rhs ) return *this;

		Release();
		m_pSmartObj = rhs.m_pSmartObj;
		AddRef();
		return *this;
	}

	template < typename T >
	SmartPointer< T > & SmartPointer< T >::operator = ( const T *pData )
	{
		if ( pData != PSX_NULL && m_pSmartObj && m_pSmartObj->GetData() == pData ) 
			return *this;

		Release();
		m_pSmartObj = pData ? new SmartData( pData ) : PSX_NULL; // ref autmatically set to 1
		return *this;
	}

	template < typename T >
	PSX_FORCE_INLINE void SmartPointer< T >::Reset(  const T *pData )
	{
		*this = pData;
	}

	template < typename T >
	PSX_FORCE_INLINE void SmartPointer< T >::Reset( void )
	{
		Release();
	}

	template < typename T >
	PSX_FORCE_INLINE const SIZE_T SmartPointer< T >::GetRefCount( void ) const
	{
		return m_pSmartObj->m_refCount;
	}

	template < typename T >
	PSX_INLINE const BOOL SmartPointer< T >::IsNull( void ) const
	{
		return m_pSmartObj == NULL || m_pSmartObj->GetData() == PSX_NULL;
	}

	/* WeakPointer Implementation */

	template < typename T >
	WeakPointer<T>::WeakPointer( void )
		: m_pSmartObj( PSX_NULL )
	{

	}

	template < typename T >
	WeakPointer< T >::WeakPointer( const SmartPointer< T > &smartPointer )
	{
		m_pSmartObj = smartPointer.m_pSmartObj;
		if ( smartPointer.IsNull() == FALSE )
			AddRef();
	}

	template < typename T >
	PSX_FORCE_INLINE WeakPointer< T >::~WeakPointer()
	{
		Release();
	}

	template < typename T >
	void WeakPointer< T >::Release()
	{
		if ( m_pSmartObj )
		{
			m_pSmartObj->ReleaseWeakRef(); 
			m_pSmartObj = NULL;
		}
	}

	template < typename T >
	PSX_INLINE void WeakPointer< T >::AddRef()
	{
		if ( m_pSmartObj )
			m_pSmartObj->AddWeakRef();
	}
	
	template <typename T >
	PSX_INLINE typename SmartPointer<T> WeakPointer<T>::Lock( void )
	{
		if ( m_pSmartObj == PSX_NULL )
			return PSX_NULL;

		return SmartPointer<T>( *m_pSmartObj );
	}
	
	template < typename T >
	PSX_INLINE BOOL WeakPointer< T >::operator == ( const WeakPointer< T > &rhs ) const
	{
		return m_pSmartObj == rhs.m_pSmartObj;
	}

	template < typename T >
	WeakPointer< T > & WeakPointer< T >::operator = ( const WeakPointer< T > &rhs )
	{
		if ( this == &rhs ) return *this;

		Release();
		m_pSmartObj = rhs.m_pSmartObj;
		AddRef();
		return *this;
	}

	template < typename T >
	WeakPointer< T > & WeakPointer< T >::operator = ( const SmartPointer< T > &smartPointer )
	{
		if ( m_pSmartObj && smartPointer.m_pSmartObj == m_pSmartObj ) return *this;

		Release();
		m_pSmartObj = smartPointer.m_pSmartObj;
		AddRef();
		return *this;
	}

	template < typename T >
	PSX_FORCE_INLINE void WeakPointer< T >::Reset( const SmartPointer< T > &smartPointer )
	{
		*this = pData;
	}

	template < typename T >
	PSX_FORCE_INLINE void WeakPointer< T >::Reset( void )
	{
		Release();
	}

	template < typename T >
	PSX_FORCE_INLINE const SIZE_T WeakPointer< T >::GetRefCount( void ) const
	{
		return m_pSmartObj ? m_pSmartObj->m_refCount : 0;
	}

	template < typename T >
	PSX_FORCE_INLINE const SIZE_T WeakPointer<T>::GetWeakCount( void ) const
	{
		return m_pSmartObj : m_pSmartObj->m_weakCount ? ;
	}

	template < typename T >
	PSX_INLINE const BOOL WeakPointer< T >::IsNull( void ) const
	{
		return m_pSmartObj ? m_pSmartObj->GetData() == PSX_NULL : PSX_NULL;
	}

	/* SmartDataArr Implementation */

	template < typename T >
	PSX_FORCE_INLINE SmartPointerArr< T >::SmartDataArr::SmartDataArr( void ) : SmartData() { }

	template < typename T >
	PSX_FORCE_INLINE SmartPointerArr< T >::SmartDataArr::SmartDataArr( const T *pData ) : SmartData( pData ) { }

	template < typename T >
	void SmartPointerArr< T >::SmartDataArr::Release()
	{
		if ( m_pData && --m_refCount == 0)
		{
			delete [] m_pData;
			delete this;
		}
	}

	/* SmartPointerArr Implementation */

	template < typename T >
	PSX_FORCE_INLINE SmartPointerArr< T >::SmartPointerArr( void ) : SmartPointer() { }

	template < typename T >
	SmartPointerArr< T >::SmartPointerArr( SmartPointerArr< T > &smartPointerArr ) : SmartPointer( smartPointerArr )
	{
		Release();
		m_pSmartObj = smartPointerArr;
		AddRef();
	}

	template < typename T >
	SmartPointerArr< T >::SmartPointerArr( T *pData ) : SmartPointer()
	{
		Release();
		m_pSmartObj = new SmartDataArr( pData );
	}

	template < typename T >
	PSX_FORCE_INLINE SmartPointerArr< T >::~SmartPointerArr()
	{
		Release();
	}

	template < typename T >
	SmartPointerArr< T > & SmartPointerArr< T >::operator = ( const T *pData )
	{
		if ( m_pSmartObj && m_pSmartObj->GetData() == pData ) return *this;

		Release();
		m_pSmartObj = new SmartDataArr( pData );
		
		return *this;
	}

	template < typename T >
	SmartPointerArr< T > & SmartPointerArr< T >::operator = ( const SmartPointerArr< T > &rhs )
	{
		if ( &rhs == this ) return *this;

		Release();
		m_pSmartObj = rhs.m_pSmartObj;
		AddRef();

		return *this;
	}



	
}