/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	StructuredBuffer.h
*
*	Description -	Represents a structured buffer in HLSL
*
*	Comments	-	
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	07/11/2013	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_STRUCTURED_BUFFER_H_
#define _PSX_STRUCTURED_BUFFER_H_

#include "PulseSTD.h"
#include "GraphicsTypes.h"
#include "ShaderResource.h"
#include "GraphicsBuffer.h"
#include "ResourcePoolItem.h"
#include "ShaderResourceViewDesc.h"
#include "UnorderedAccessViewDesc.h"

namespace Pulse
{
	struct StructuredBufferDesc : public BaseResDesc
	{
		StructuredBufferDesc( void );

		void SetDefault( void );
		
		void Set( SIZE_T structureSize, SIZE_T numElements, EBufferUsage::Type usage, void *pInitialData = PSX_NULL );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::STRUCTURED_BUFFER); }

		SIZE_T				m_structureSize;
		SIZE_T				m_numElements;
		EBufferUsage::Type	m_usage;
		void				*m_pData;

		ShaderResourceViewDesc	m_srvDesc;
		UnorderedAccessViewDesc m_uavDesc;
	};

	class StructuredBuffer : public IResourcePoolItem, public ShaderResource, public GraphicsBuffer
	{
	public:

		virtual ~StructuredBuffer( void );

		// These defines the base interface for all derived resources
		virtual BOOL CreateResource( BaseResDesc *pDesc );
		virtual BOOL DestroyResource( void );
		virtual BOOL LoadResource( void );
		virtual BOOL UnloadResource( void );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::STRUCTURED_BUFFER); }

	public:

	private:

		StructuredBuffer( void );

		EErrorCode::Type Initialize( void );

		void Cleanup( void );

	private:

		PSX_MakeStoragePoolFriend( StructuredBuffer );

		StructuredBufferDesc	m_desc;
	};
}

#endif /* _PSX_STRUCTURED_BUFFER_H_ */