/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	VertexBuffer.h
*
*	Description -	
*
*	Comments	-	
*
*	Modification History:
*			Name			   Date					Description
*			MrCodeSushi	-	11/05/2011	-	Creation of this file
**************************************************************************************/
#ifndef _PSX_VERTEX_BUFFER_H_
#define _PSX_VERTEX_BUFFER_H_

#include "PulseSTD.h"
#include "GraphicsTypes.h"
#include "ResourcePoolItem.h"
#include "ShaderResource.h"
#include "GraphicsBuffer.h"
#include "String.h"

namespace Pulse
{
	struct VertexBufferDesc : public BaseResDesc
	{
		VertexBufferDesc( void );

		void SetAsDynamicVertexBuffer( SIZE_T32 _vertexSize, SIZE_T32 _numVertices, void *pVertexData = PSX_NULL );
		
		void SetAsStaticVertexBuffer( SIZE_T32 _vertexSize, SIZE_T32 _numVertices, void *pVertexData );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::VERTEX_BUFFER ); }

		EVertexBufferType::Type bufferType;
		SIZE_T32			vertexSize;
		SIZE_T32			numVertices;
		SIZE_T32			numPrimitives;
		void				*pData;	// This is not managed by the VertexBuffer nor the desc.
	};

	class VertexBuffer : public IResourcePoolItem, /*public ShaderResource,*/ public GraphicsBuffer
	{
	public:

		virtual ~VertexBuffer( void );

		// These defines the base interface for all derived resources
		virtual BOOL CreateResource( BaseResDesc *pDesc );
		virtual BOOL DestroyResource( void );
		virtual BOOL LoadResource( void );	
		virtual BOOL UnloadResource( void );

		virtual const ResourceCode GetType( void ) const { return ResourceCode( EResource::VIDEO, EVideoResource::VERTEX_BUFFER ); }

		void * GetNakedVertexBuffer( void ) { return (void*)GetNakedBuffer(); }

		VertexBufferDesc * GetDescription( void ) { return &m_desc; }

	private:

		VertexBuffer( void );

		EErrorCode::Type Initialize( void );

		void Cleanup( void );

	private:

		PSX_MakeStoragePoolFriend( VertexBuffer );

		VertexBufferDesc	m_desc;

	};
}

#endif /* _PSX_VERTEX_BUFFER_H_ */