/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	ConstantBuffer.cpp
*
*	Comments	-	See ConstantBuffer.h
*
**************************************************************************************/
#include "../Include/ConstantBuffer.h"
#include "../Include/Engine.h"
#include "../Include/AsyncLoadResource.h"

namespace Pulse
{
	ConstantBufferDesc::ConstantBufferDesc( void )
	{
		bufferSize = 0;
		bufferUsage = EBufferUsage::UNKNOWN;
		pData = PSX_NULL;
	}

	void ConstantBufferDesc::SetAsDynamicConstantBuffer( SIZE_T _bufferSize )
	{
		bufferSize = _bufferSize;
		bufferUsage = EBufferUsage::DYNAMIC;
	}

	void ConstantBufferDesc::SetAsStaticConstantBuffer( SIZE_T _bufferSize, void *pInitialData )
	{
		bufferSize = _bufferSize;
		bufferUsage = EBufferUsage::STATIC;
		pData = pInitialData;
	}

	ConstantBuffer::ConstantBuffer( void )
	{
	}

	ConstantBuffer::~ConstantBuffer( void )
	{

	}

	BOOL ConstantBuffer::CreateResource( BaseResDesc *pDesc )	
	{
		if ( pDesc->GetType() != GetType() )
			return FALSE;

		if ( ((ConstantBufferDesc*)pDesc)->bufferUsage == EBufferUsage::STATIC &&
			((ConstantBufferDesc*)pDesc)->pData == PSX_NULL )
		{
			PSX_PushError( "Can't create static constant buffer without providing an initial data." );
			return FALSE;
		}

		m_desc = *((ConstantBufferDesc*)pDesc);

		if ( PerformInitialize( m_desc.IsAsyncLoad() ) == EErrorCode::OKAY )
			SetFlagCreated();

		return IsCreated();	
	}

	BOOL ConstantBuffer::DestroyResource( void )				
	{
		UnloadResource();
		SetFlagDestroyed();	
		return TRUE;	
	}

	BOOL ConstantBuffer::LoadResource( void )				
	{ 
		if ( Initialize() == EErrorCode::OKAY )
		{
			SetFlagLoaded();	
			return TRUE;	
		}

		return FALSE;
	}

	BOOL ConstantBuffer::UnloadResource( void )				
	{
		Cleanup();
		SetFlagUnloaded();
		return FALSE;	
	}

	EErrorCode::Type ConstantBuffer::Initialize( void )
	{
		Renderer *pRenderer = Engine::GetInstance()->GetRenderer();
		ID3D11Device *pDevice = (ID3D11Device*)pRenderer->GetNakedDevice();

		HRESULT hr;
		D3D11_BUFFER_DESC desc;

		desc.ByteWidth				= (UINT)m_desc.bufferSize;
		desc.BindFlags				= D3D11_BIND_CONSTANT_BUFFER;
		desc.MiscFlags				= 0;
		desc.StructureByteStride	= 0;

		if ( m_desc.bufferUsage == EBufferUsage::DYNAMIC )
		{
			desc.Usage = D3D11_USAGE_DYNAMIC;
			desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
		}
		else if ( m_desc.bufferUsage == EBufferUsage::STATIC )
		{
			desc.Usage = D3D11_USAGE_IMMUTABLE;
			desc.CPUAccessFlags = 0;
		}
		else
		{
			PSX_PushError( "Not yet implemented." );
		}

		if ( m_desc.pData )
		{
			D3D11_SUBRESOURCE_DATA data;
			data.pSysMem = m_desc.pData;
			data.SysMemPitch = 0;
			data.SysMemSlicePitch = 0;

			hr = pDevice->CreateBuffer( &desc, &data, &m_pBuffer );
		}
		else
			hr = pDevice->CreateBuffer( &desc, PSX_NULL, &m_pBuffer );

		m_desc.pData = PSX_NULL;

		if ( FAILED(hr) )
		{
			PSX_PushError( "Failed to create D3D11 constant buffer." );
			return EErrorCode::GRAPHICS;
		}

		//CreateResourceViews( m_pBuffer, desc.BindFlags );

		return EErrorCode::OKAY;
	}

	void ConstantBuffer::Cleanup( void )
	{
		//ReleaseResourceViews();
		ReleaseBuffer();
	}
}