/**************************************************************************************
*
*	Copyright (C) 2009 - 2012 Brent Andrew O. Chua - See LICENSE.txt for more info.
*		
*	Filename	-	VertexShaderStage.cpp
*
*	Comments	-	See VertexShaderStage.h
*
**************************************************************************************/
#include "../Include/VertexShaderStage.h"
#include "../Include/Renderer.h"
#include "../Include/VertexShader.h"

namespace Pulse
{
	VertexShaderStage::VertexShaderStage( void )
		: IShaderStage()
	{

	}

	VertexShaderStage::~VertexShaderStage( void )
	{

	}

	void VertexShaderStage::ClearStates( void )
	{
		m_stateCache.ClearStates();
	}
	
	void VertexShaderStage::BindStates( void )
	{
		BindShader();
		BindConstantBuffers();
		BindShaderResourceViews();
		BindSamplerStates();
	}

	void VertexShaderStage::BindShader( void )
	{
		ID3D11DeviceContext *pDC = (ID3D11DeviceContext*)m_pRenderer->GetNakedDeviceContext();

		if ( m_stateCache.m_pShader )
		{
			ID3D11VertexShader *pShader = (ID3D11VertexShader*)m_stateCache.m_pShader->GetNakedShader();

			if ( m_stateCache.m_pShader->GetShaderType() == EShader::VERTEX )
				pDC->VSSetShader( pShader, 0, 0 );
			else
				PSX_PushError( "Invalid shader type. Must bspecify a vertex shader type." );
		}
		else
		{
			pDC->VSSetShader( PSX_NULL, 0, 0 );
		}
	}
	
	void VertexShaderStage::BindConstantBuffers( void )
	{
		ID3D11DeviceContext *pDC = (ID3D11DeviceContext*)m_pRenderer->GetNakedDeviceContext();
		pDC->VSSetConstantBuffers( 0, PSX_NUM_CONSTANT_BUFFER_SLOT, (ID3D11Buffer * const *)m_stateCache.m_pD3DConstantBuffers );
	}
	
	void VertexShaderStage::BindSamplerStates( void )
	{
		ID3D11DeviceContext *pDC = (ID3D11DeviceContext*)m_pRenderer->GetNakedDeviceContext();
		pDC->VSSetSamplers( 0,PSX_NUM_SAMPLER_SLOT, (ID3D11SamplerState * const *)m_stateCache.m_pD3DSamplerStates );
	}
	
	void VertexShaderStage::BindShaderResourceViews( void )
	{
		ID3D11DeviceContext *pDC = (ID3D11DeviceContext*)m_pRenderer->GetNakedDeviceContext();
		pDC->VSSetShaderResources( 0, PSX_NUM_INPUT_RESOURCE_SLOT, (ID3D11ShaderResourceView * const *)m_stateCache.m_pD3DShaderResources );
	}
}